!------------------------------------------------------------------------------
! Cumulus Parameterization by G. Grell
!-Convective transport for smoke aerosols and non-hygroscopic gases
!-Developed by Saulo Freitas (sfreitas@cptec.inpe.br)
!-ref: Freitas, S.R., et al.: Monitoring the transport of biomass burning
!      emissions in South America. Environmental Fluid Mechanics,
!      Kluwer Academic Publishers, 2005.
!------------------------------------------------------------------------------

subroutine CUPARM_GRELL_CATT(iens)

  use mem_basic         , only: basic_g
  use mem_tend          , only: tend
  use mem_cuparm        , only: confrq,cuparm_g,cuparm_g_sh

  use node_mod          , only: mynum,   &   ! INTENT(IN)
             		        mxp,     &   ! INTENT(IN)
             		        myp,     &   ! INTENT(IN)
             		        mzp,     &   ! INTENT(IN)
             		        ia,      &   ! INTENT(IN)
             		        iz,      &   ! INTENT(IN)
             		        ja,      &   ! INTENT(IN)
             		        jz,      &   ! INTENT(IN)
             		        i0,      &   ! INTENT(IN)  
             		        j0	     ! INTENT(IN) 
  use mem_grid          , only: time,    &   ! INTENT(IN)
            		   	initial, &   ! INTENT(IN)
            		   	dtlt,	 &   ! INTENT(IN)
            		   	itime1,  &   ! INTENT(IN)
            		   	ngrid,   &   ! INTENT(IN)
            		   	grid_g,  &   ! INTENT(IN)  	  
            		   	dtlongn, &   ! INTENT(IN)
           		   	deltaxn, &   ! INTENT(IN)
           		   	deltayn, &
				npatch
				

  use rconstants        , only: tkmin 
  use extras            , only: extra3d,extra2d 
  use mem_turb          , only: turb_g
  use mem_micro         , only: micro_g
  use mem_scratch       , only: scratch
  use mem_scalar        , only: scalar_g
!srf
  use io_params         , only: frqanl
  use mem_leaf          , only: leaf_g
  use micphys           , only: level

!- use modules for Grell Parameterization
  use mem_grell_param   , only: mgmxp,mgmyp,mgmzp,maxiens,ngrids_cp
  use mem_scratch1_grell, only: ierr4d,jmin4d,kdet4d,k224d,kbcon4d,ktop4d,kpbl4d,kstabi4d, &
                                kstabm4d,xmb4d,edt4d,enup5d,endn5d,deup5d,dedn5d,zup5d,    &
                                zdn5d,iruncon,zcup5d,pcup5d,prup5d,clwup5d,tup5d
  use mem_grell         , only: grell_g,grell_g_sh
  
  implicit none

  integer, intent(IN) :: iens
  integer,parameter :: CPTIME = 0. !orig: CPTIME = 7200.

!!$  integer,parameter :: i_forcing = 1


  !------------------------ deep convection -----------------------------------
  if(iens == 1) then 
  
  !
  !        Zero out tendencies initially
     if (TIME.eq.0.) then 
        call azero(mxp*myp*mzp,cuparm_g(ngrid)%thsrc(1,1,1))
        call azero(mxp*myp*mzp,cuparm_g(ngrid)%rtsrc(1,1,1))
        call azero(mxp*myp*mzp,extra3d(1,ngrid)%d3(1,1,1)  )
        call azero(mxp*myp    ,extra2d(3,ngrid)%d2(1,1)    )  
     end if 

     !_old : IF(INITIAL.EQ.2.AND.TIME.LT.CPTIME)RETURN
     if(INITIAL.eq.2.and.TIME.lt.CPTIME-dtlt) return
     !_old : IF(MOD(TIME+DTLT+.001,CONFRQ).LE.DTLT.OR.time.lt..01)THEN
     if(mod(TIME,CONFRQ).lt.DTLT.or.time.lt. .01 .or.abs(time-cptime) .lt. 0.01) then !002

        iruncon=1
        
!        print 90,TIME+DTLT,(TIME+DTLT)/3600. +(itime1/100+mod(itime1,100)/60.)
!90      format('   deep conv tendencies updated    time =',F10.1,  &
!             '  real time (hrs) =',F6.1)

        call azero(mxp*myp*mzp,cuparm_g(ngrid)%thsrc(1,1,1))
        call azero(mxp*myp*mzp,cuparm_g(ngrid)%rtsrc(1,1,1))
        call azero(mxp*myp    ,cuparm_g(ngrid)%conprr(1,1) )
        call azero(mxp*myp*mzp,extra3d(1,ngrid)%d3(1,1,1)  )  
        call azero(mxp*myp*mzp,extra3d(5,ngrid)%d3(1,1,1)  )!cloud/ice tendency
        call azero(mxp*myp    ,extra2d(3,ngrid)%d2(1,1)    )  

        ! >>>> if(mod(TIME-DTLT,frqanl).lt.DTLT) then
        if(mod(time,frqanl) < dtlongn(1) ) then
	     !print*,'---- zerout extra2d array time=',time
             call azero(mxp*myp*mzp,extra3d(4,ngrid)%d3(1,1,1)  )  
	endif

!!$        !srf - use the old way to define the cumulus forcing
!!$	if(i_forcing /= 1) then
!!$             call atob(mxp * myp * mzp,tend%THT(1),grell_g(ngrid)%lsfth(1,1,1))
!!$             call atob(mxp * myp * mzp,tend%RTT(1),grell_g(ngrid)%lsfrt(1,1,1))
!!$        endif	

        call cuparth_catt(                 &
             mynum,                        & !01
             mgmxp,                        & !02
             mgmyp,                        & !03
             mgmzp,                        & !04
             mzp,                          & !05
             mxp,                          & !06
             myp,                          & !07
             ia,                           & !08
             iz,                           & !09
             ja,                           & !10
             jz,                           & !11
             i0,                           & !12
             j0,                           & !13
             maxiens,                      & !14
             iens,                         & !15
             ngrid,                        & !16
             ngrids_cp,                    & !17
             DTLT,                         & !18
             time,                         & !19
             !
             basic_g(ngrid)%up   (1,1,1),    & !20
             basic_g(ngrid)%vp   (1,1,1),    & !21
             basic_g(ngrid)%wp   (1,1,1),    & !22
             basic_g(ngrid)%theta(1,1,1),    & !23
             basic_g(ngrid)%pp   (1,1,1),    & !24
             basic_g(ngrid)%pi0  (1,1,1),    & !25
             basic_g(ngrid)%dn0  (1,1,1),    & !26
             basic_g(ngrid)%rv   (1,1,1),    & !27
             turb_g(ngrid)%tkep  (1,1,1),    & !28 *
             tkmin,                          & !29 *
             micro_g(ngrid)%rcp  (1,1,1),    & !30 *
	     grid_g(ngrid)%topt  (1,1),      & !31
             grid_g(ngrid)%RTGT  (1,1),      & !32
             !
             !grell_g(ngrid)%lsfth(1,1,1)  ,  & !33 
             !grell_g(ngrid)%lsfrt(1,1,1)  ,  & !34 
             !
             tend%THT(1),                    & !33 
             tend%RTT(1),                    & !34 
             !
             tend%PT(1),                     & !35
             cuparm_g(ngrid)%THSRC (1,1,1),  & !36 
             cuparm_g(ngrid)%RTSRC (1,1,1),  & !37 
             cuparm_g(ngrid)%CONPRR(1,1),    & !38	
             extra3d(1,ngrid)%d3   (1,1,1),  & !39
             extra3d(4,ngrid)%d3   (1,1,1),  & !40
             extra3d(5,ngrid)%d3   (1,1,1),  & !41
             extra2d(3,ngrid)%d2   (1,1),    & !42
             !
             ierr4d,                       & !43
             jmin4d,                       & !44
             kdet4d,                       & !45
             k224d,                        & !46
             kbcon4d,                      & !47
             ktop4d,                       & !48
             kpbl4d,                       & !49
             kstabi4d,                     & !50
             kstabm4d,                     & !51
             xmb4d,                        & !52
             edt4d,                        & !53
             !
             zcup5d,                       & !54 
             pcup5d,                       & !55 
             enup5d,                       & !56
             endn5d,                       & !57
             deup5d,                       & !58 
             dedn5d,                       & !59
             zup5d,                        & !60
             zdn5d ,                       & !61
             prup5d,                       & !62
             clwup5d,                      & !63
             tup5d ,                       & !64
             !
             grell_g(ngrid)%upmf   (1,1),  & !65
             grell_g(ngrid)%dnmf   (1,1),  & !66
             grell_g(ngrid)%xierr  (1,1),  & !67
             grell_g(ngrid)%xktop  (1,1),  & !68
             grell_g(ngrid)%xkbcon (1,1),  & !69
             grell_g(ngrid)%xk22   (1,1),  & !70
             grell_g(ngrid)%xjmin  (1,1),  & !71
             grell_g(ngrid)%xkdt   (1,1),  & !72
             grell_g(ngrid)%xiact_p(1,1),  & !73
             grell_g(ngrid)%xiact_c(1,1),  & !74
	     confrq,                       & !75
             frqanl,                       & !76
	     deltaxn(ngrid)*deltayn(ngrid),& !77
             leaf_g(ngrid)%patch_area(1,1,1), &!78
	     npatch,                       & !79
	     level)                          !80

     end if 
     call accum(mxp*myp*mzp, tend%tht(1), cuparm_g(ngrid)%thsrc(1,1,1))
     call accum(mxp*myp*mzp, tend%rtt(1), cuparm_g(ngrid)%rtsrc(1,1,1))
     call update(mxp*myp, cuparm_g(ngrid)%aconpr(1,1),cuparm_g(ngrid)%conprr(1,1),dtlt)

  endif 

  !---------------   Shallow cumulus scheme -----------------------------------
  if(iens == 2) then !006
     if(TIME.eq.0.) then !004
        call azero(mxp*myp*mzp,cuparm_g_sh(ngrid)%thsrc(1,1,1))
        call azero(mxp*myp*mzp,cuparm_g_sh(ngrid)%rtsrc(1,1,1))
     end if 

     if(INITIAL.eq.2.and.TIME.lt.CPTIME-dtlt) return
     if(mod(TIME,CONFRQ).lt.DTLT.or.time.lt. .01 .or. abs(time-cptime).lt. 0.01) then !005
        iruncon=1

!        print 91,TIME+DTLT,(TIME+DTLT)/3600.+(itime1/100+mod(itime1,100)/60.)
!91      format('   shallow conv tendencies updated time =',f10.1,  &
!               ' real time (hrs) =',F6.1)

        call azero(mxp*myp*mzp,cuparm_g_sh(ngrid)%thsrc(1,1,1))
        call azero(mxp*myp*mzp,cuparm_g_sh(ngrid)%rtsrc(1,1,1))           
        call azero(mxp*myp    ,extra2d(2,ngrid)%d2(1,1)       )  


        call cuparth_shal(       &
             mynum,      &       !01
             mgmxp,      &	 !02
             mgmyp,      &	 !03
             mgmzp,      &	 !04
             mzp,        &	 !05
             mxp,        &	 !06
             myp,        &	 !07
             ia,         &	 !08
             iz,         &	 !09
             !
             ja,         &	 !10
             jz,         &	 !11
             i0,         &	 !12
             j0,         &	 !13
             maxiens,    &	 !15
             iens,       &	 !16
             ngrid,      &	 !17
             ngrids_cp,  &	 !18
             dtlt,       &	 !19
             time,       &       !20
             !
             basic_g(ngrid)%up   (1,1,1),     &   !21
             basic_g(ngrid)%vp   (1,1,1),     &   !22
             basic_g(ngrid)%wp   (1,1,1),     &   !23
             basic_g(ngrid)%theta(1,1,1),     &   !24
             basic_g(ngrid)%pp   (1,1,1),     &   !25
             basic_g(ngrid)%pi0  (1,1,1),     &   !26
             basic_g(ngrid)%dn0  (1,1,1),     &   !27
             basic_g(ngrid)%rv   (1,1,1),     &   !28
             turb_g(ngrid)%tkep  (1,1,1),     &   !29
             !
             tkmin,                          &   !30
             micro_g(ngrid)%rcp     (1,1,1), &   !31
             grid_g(ngrid)%topt     (1,1),   &   !32
             grid_g(ngrid)%RTGT     (1,1),   &   !33
             !
             !grell_g_sh(ngrid)%lsfth(1,1,1), &   !34
             !grell_g_sh(ngrid)%lsfrt(1,1,1), &   !35
             !
             tend%THT(1),                    &   !34
             tend%RTT(1),                    &   !35
             !
             tend%PT(1),		     &   !36
             cuparm_g_sh(ngrid)%thsrc(1,1,1),&   !37
             cuparm_g_sh(ngrid)%rtsrc(1,1,1),&   !38
             extra3d(2,ngrid)%d3     (1,1,1),&   !39 !<< usando extra3d(2)
             extra2d(2,ngrid)%d2     (1,1),  &   !39 !<< usando extra2d(2)
             !
             ierr4d,                         & !40
             jmin4d,                         & !41
             kdet4d,                         & !42
             k224d,                          & !43
             kbcon4d,                        & !44
             ktop4d,                         & !45
             kpbl4d,                         & !46
             kstabi4d,                       & !47
             kstabm4d,                       & !48
             !
             xmb4d,                          & !49
             edt4d,                          & !50
             zcup5d,                         & !51
             pcup5d,                         & !52
             enup5d,                         & !53
             endn5d,                         & !54
             deup5d,                         & !55
             dedn5d,                         & !56
             zup5d,                          & !57
             zdn5d,                          & !58
             prup5d,                         & !59
             clwup5d,                        & !60
             tup5d,                          & !61
             !
             grell_g_sh(ngrid)%upmf  (1,1),  & !62
             grell_g_sh(ngrid)%xierr (1,1),  & !63
             grell_g_sh(ngrid)%xktop (1,1),  & !64
             grell_g_sh(ngrid)%xkbcon(1,1),  & !65
             grell_g_sh(ngrid)%xk22  (1,1),  & !66
             grell_g   (ngrid)%xierr (1,1),  & !67
	     confrq,frqanl,                  &
	     deltaxn(ngrid)*deltayn(ngrid),  &
             leaf_g(ngrid)%patch_area(1,1,1),&
	     npatch,                         &
	     level)

     end if !005
     call accum(mxp*myp*mzp, tend%tht(1), cuparm_g_sh(ngrid)%thsrc(1,1,1))
     call accum(mxp*myp*mzp, tend%rtt(1), cuparm_g_sh(ngrid)%rtsrc(1,1,1))

  end if !006

  !--------- Convective Transport based on mass flux scheme -------------------
  !
  !go to 2000   !desvia do transporte convectivo

  if(iruncon == 1) then
 
     call azero(mxp*myp*mzp,scratch%scr1(1)) 

     call trans_conv_mflx(iens,scratch%scr1(1))

     if(iens.eq.2) call AE1M1(mxp*myp*mzp,extra3d(3,ngrid)%d3(1,1,1), &
                              scalar_g(1,ngrid)%sclt(1),              &
			      extra3d(3,ngrid)%d3(1,1,1)) 
  end if 
2000 continue

end subroutine CUPARM_GRELL_CATT
